@@ -31,6 +31,7 @@
 #include "gtksourcebuffer-private.h"
 
 #include <string.h>
+#include <stdlib.h>
 #include <gtk/gtk.h>
 
 #include "gtksourcelanguage.h"
@@ -100,12 +101,24 @@
  * engine. There are currently three default context classes that are
  * applied to regions of a #GtkSourceBuffer:
  *  - <emphasis>comment</emphasis>: the region delimits a comment;
- *  - <emphasis>string</emphasis>: the region delimits a string;
- *  - <emphasis>no-spell-check</emphasis>: the region should not be spell checked.
+ *  - <emphasis>no-spell-check</emphasis>: the region should not be spell checked;
+ *  - <emphasis>path</emphasis>: the region delimits a path to a file;
+ *  - <emphasis>string</emphasis>: the region delimits a string.
  *
  * Custom language definition files can create their own context classes,
  * since the functions like gtk_source_buffer_iter_has_context_class() take
  * a string parameter as the context class.
+ *
+ * Each context class has an associated #GtkTextTag with the name
+ * <emphasis>gtksourceview:context-classes:&lt;name&gt;</emphasis>. For example to
+ * retrieve the #GtkTextTag for the string context class, one can write:
+ * |[
+ * GtkTextTagTable *tag_table;
+ * GtkTextTag *tag;
+ *
+ * tag_table = gtk_text_buffer_get_tag_table (buffer);
+ * tag = gtk_text_tag_table_lookup (tag_table, "gtksourceview:context-classes:string");
+ * ]|
  */
 
 /*
@@ -130,7 +143,7 @@
 /* For bracket matching */
 #define MAX_CHARS_BEFORE_FINDING_A_MATCH    10000
 
-#define TAG_CONTEXT_CLASS_NAME "GtkSourceViewTagContextClassName"
+#define CONTEXT_CLASSES_PREFIX "gtksourceview:context-classes:"
 
 /* Signals */
 enum
@@ -2139,6 +2152,23 @@ gtk_source_buffer_remove_source_marks (GtkSourceBuffer   *buffer,
 	g_slist_free (list);
 }
 
+static GtkTextTag *
+get_context_class_tag (GtkSourceBuffer *buffer,
+		       const gchar     *context_class)
+{
+	gchar *tag_name;
+	GtkTextTagTable *tag_table;
+	GtkTextTag *tag;
+
+	tag_name = g_strdup_printf (CONTEXT_CLASSES_PREFIX "%s", context_class);
+
+	tag_table = gtk_text_buffer_get_tag_table (GTK_TEXT_BUFFER (buffer));
+	tag = gtk_text_tag_table_lookup (tag_table, tag_name);
+
+	g_free (tag_name);
+	return tag;
+}
+
 /**
  * gtk_source_buffer_iter_has_context_class:
  * @buffer: a #GtkSourceBuffer.
@@ -2163,22 +2193,14 @@ gtk_source_buffer_iter_has_context_class (GtkSourceBuffer   *buffer,
 	g_return_val_if_fail (iter != NULL, FALSE);
 	g_return_val_if_fail (context_class != NULL, FALSE);
 
-	if (buffer->priv->highlight_engine == NULL)
-	{
-		return FALSE;
-	}
-
-	tag = _gtk_source_engine_get_context_class_tag (buffer->priv->highlight_engine,
-							context_class);
+	tag = get_context_class_tag (buffer, context_class);
 
 	if (tag != NULL)
 	{
 		return gtk_text_iter_has_tag (iter, tag);
 	}
-	else
-	{
-		return FALSE;
-	}
+
+	return FALSE;
 }
 
 /**
@@ -2200,8 +2222,9 @@ gchar **
 gtk_source_buffer_get_context_classes_at_iter (GtkSourceBuffer   *buffer,
                                                const GtkTextIter *iter)
 {
+	const gsize prefix_len = strlen (CONTEXT_CLASSES_PREFIX);
 	GSList *tags;
-	GSList *item;
+	GSList *l;
 	GPtrArray *ret;
 
 	g_return_val_if_fail (GTK_SOURCE_IS_BUFFER (buffer), NULL);
@@ -2210,15 +2233,22 @@ gtk_source_buffer_get_context_classes_at_iter (GtkSourceBuffer   *buffer,
 	tags = gtk_text_iter_get_tags (iter);
 	ret = g_ptr_array_new ();
 
-	for (item = tags; item != NULL; item = g_slist_next (item))
+	for (l = tags; l != NULL; l = l->next)
 	{
-		gchar const *name = g_object_get_data (G_OBJECT (item->data),
-		                                       TAG_CONTEXT_CLASS_NAME);
+		GtkTextTag *tag = l->data;
+		gchar *tag_name;
 
-		if (name != NULL)
+		g_object_get (tag, "name", &tag_name, NULL);
+
+		if (tag_name != NULL &&
+		    g_str_has_prefix (tag_name, CONTEXT_CLASSES_PREFIX))
 		{
-			g_ptr_array_add (ret, g_strdup (name));
+			gchar *context_class_name = g_strdup (tag_name + prefix_len);
+
+			g_ptr_array_add (ret, context_class_name);
 		}
+
+		g_free (tag_name);
 	}
 
 	g_slist_free (tags);
@@ -2256,22 +2286,14 @@ gtk_source_buffer_iter_forward_to_context_class_toggle (GtkSourceBuffer *buffer,
 	g_return_val_if_fail (iter != NULL, FALSE);
 	g_return_val_if_fail (context_class != NULL, FALSE);
 
-	if (buffer->priv->highlight_engine == NULL)
-	{
-		return FALSE;
-	}
+	tag = get_context_class_tag (buffer, context_class);
 
-	tag = _gtk_source_engine_get_context_class_tag (buffer->priv->highlight_engine,
-							context_class);
-
-	if (tag == NULL)
-	{
-		return FALSE;
-	}
-	else
+	if (tag != NULL)
 	{
 		return gtk_text_iter_forward_to_tag_toggle (iter, tag);
 	}
+
+	return FALSE;
 }
 
 /**
@@ -2303,22 +2325,14 @@ gtk_source_buffer_iter_backward_to_context_class_toggle (GtkSourceBuffer *buffer
 	g_return_val_if_fail (iter != NULL, FALSE);
 	g_return_val_if_fail (context_class != NULL, FALSE);
 
-	if (buffer->priv->highlight_engine == NULL)
-	{
-		return FALSE;
-	}
+	tag = get_context_class_tag (buffer, context_class);
 
-	tag = _gtk_source_engine_get_context_class_tag (buffer->priv->highlight_engine,
-							context_class);
-
-	if (tag == NULL)
-	{
-		return FALSE;
-	}
-	else
+	if (tag != NULL)
 	{
 		return gtk_text_iter_backward_to_tag_toggle (iter, tag);
 	}
+
+	return FALSE;
 }
 
 /*
@@ -2696,6 +2710,183 @@ gtk_source_buffer_join_lines (GtkSourceBuffer *buffer,
 	gtk_text_buffer_delete_mark (text_buffer, end_mark);
 }
 
+static gchar *
+get_line_slice (GtkTextBuffer *buf,
+		gint           line)
+{
+	GtkTextIter start, end;
+
+	gtk_text_buffer_get_iter_at_line (buf, &start, line);
+	end = start;
+
+	if (!gtk_text_iter_ends_line (&start))
+	{
+		gtk_text_iter_forward_to_line_end (&end);
+	}
+
+	return gtk_text_buffer_get_slice (buf, &start, &end, TRUE);
+}
+
+typedef struct {
+	gchar *line; /* the original text to re-insert */
+	gchar *key;  /* the key to use for the comparison */
+} SortLine;
+
+static gint
+compare_line (gconstpointer aptr,
+              gconstpointer bptr)
+{
+	const SortLine *a = aptr;
+	const SortLine *b = bptr;
+
+	return g_strcmp0 (a->key, b->key);
+}
+
+static gint
+compare_line_reversed (gconstpointer aptr,
+                       gconstpointer bptr)
+{
+	const SortLine *a = aptr;
+	const SortLine *b = bptr;
+
+	return g_strcmp0 (b->key, a->key);
+}
+
+/**
+ * gtk_source_buffer_sort_lines:
+ * @buffer: a #GtkSourceBuffer.
+ * @start: a #GtkTextIter.
+ * @end: a #GtkTextIter.
+ * @flags: #GtkSourceSortFlags specifying how the sort should behave
+ * @column: sort considering the text starting at the given column
+ *
+ * Sort the lines of text between the specified iterators.
+ *
+ * Since: 3.18
+ */
+void
+gtk_source_buffer_sort_lines (GtkSourceBuffer    *buffer,
+                              GtkTextIter        *start,
+                              GtkTextIter        *end,
+                              GtkSourceSortFlags  flags,
+                              gint                column)
+{
+	GtkTextBuffer *text_buffer;
+	gint start_line;
+	gint end_line;
+	gint num_lines;
+	SortLine *lines;
+	gchar *last_line = NULL;
+	gint i;
+
+	g_return_if_fail (GTK_SOURCE_IS_BUFFER (buffer));
+	g_return_if_fail (start != NULL);
+	g_return_if_fail (end != NULL);
+
+	text_buffer = GTK_TEXT_BUFFER (buffer);
+
+	gtk_text_iter_order (start, end);
+
+	start_line = gtk_text_iter_get_line (start);
+	end_line = gtk_text_iter_get_line (end);
+
+	/* if we are at line start our last line is the previus one.
+	 * Otherwise the last line is the current one but we try to
+	 * move the iter after the line terminator */
+	if (gtk_text_iter_get_line_offset (end) == 0)
+	{
+		end_line = MAX (start_line, end_line - 1);
+	}
+	else
+	{
+		gtk_text_iter_forward_line (end);
+	}
+
+	num_lines = end_line - start_line + 1;
+	lines = g_new0 (SortLine, num_lines);
+
+	for (i = 0; i < num_lines; i++)
+	{
+		gchar *line;
+		gboolean free_line = FALSE;
+		glong length;
+
+		lines[i].line = get_line_slice (text_buffer, start_line + i);
+
+		if ((flags & GTK_SOURCE_SORT_FLAGS_CASE_SENSITIVE) != 0)
+		{
+			line = lines[i].line;
+		}
+		else
+		{
+			line = g_utf8_casefold (lines[i].line, -1);
+			free_line = TRUE;
+		}
+
+		length = g_utf8_strlen (line, -1);
+
+		if (length < column)
+		{
+			lines[i].key = NULL;
+		}
+		else if (column > 0)
+		{
+			gchar *substring;
+
+			substring = g_utf8_offset_to_pointer (line, column);
+			lines[i].key = g_utf8_collate_key (substring, -1);
+		}
+		else
+		{
+			lines[i].key = g_utf8_collate_key (line, -1);
+		}
+
+		if (free_line)
+		{
+			g_free (line);
+		}
+	}
+
+	if ((flags & GTK_SOURCE_SORT_FLAGS_REVERSE_ORDER) != 0)
+	{
+		qsort (lines, num_lines, sizeof (SortLine), compare_line_reversed);
+	}
+	else
+	{
+		qsort (lines, num_lines, sizeof (SortLine), compare_line);
+	}
+
+	_gtk_source_buffer_save_and_clear_selection (buffer);
+	gtk_text_buffer_begin_user_action (text_buffer);
+
+	gtk_text_buffer_delete (text_buffer, start, end);
+
+	for (i = 0; i < num_lines; i++)
+	{
+		if ((flags & GTK_SOURCE_SORT_FLAGS_REMOVE_DUPLICATES) != 0 &&
+		    g_strcmp0 (last_line, lines[i].line) == 0)
+		{
+			continue;
+		}
+
+		gtk_text_buffer_insert (text_buffer, start, lines[i].line, -1);
+		gtk_text_buffer_insert (text_buffer, start, "\n", -1);
+
+		last_line = lines[i].line;
+	}
+
+	gtk_text_buffer_end_user_action (text_buffer);
+	_gtk_source_buffer_restore_selection (buffer);
+
+	for (i = 0; i < num_lines; i++)
+	{
+		g_free (lines[i].line);
+		g_free (lines[i].key);
+	}
+
+	g_free (lines);
+}
+
 /**
  * gtk_source_buffer_set_undo_manager:
  * @buffer: a #GtkSourceBuffer.
