@@ -43,6 +43,18 @@
  * A file loader should be used only for one load operation, including errors
  * handling. If an error occurs, you can reconfigure the loader and relaunch the
  * operation with gtk_source_file_loader_load_async().
+ *
+ * Running a #GtkSourceFileLoader is an undoable action for the
+ * #GtkSourceBuffer. That is, gtk_source_buffer_begin_not_undoable_action() and
+ * gtk_source_buffer_end_not_undoable_action() are called, which delete the
+ * undo/redo history.
+ *
+ * After a file loading, the buffer is reset to the contents provided by the
+ * #GFile or #GInputStream, so the buffer is set as “unmodified”, that is,
+ * gtk_text_buffer_set_modified() is called with %FALSE. If the contents isn't
+ * saved somewhere (for example if you load from stdin), then you should
+ * probably call gtk_text_buffer_set_modified() with %TRUE after calling
+ * gtk_source_file_loader_load_finish().
  */
 
 #if 0
@@ -1090,7 +1102,7 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 				    GError              **error)
 {
 	gboolean ok;
-	gboolean update_buffer_properties;
+	gboolean update_file_properties;
 	GError *real_error = NULL;
 
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE_LOADER (loader), FALSE);
@@ -1112,11 +1124,11 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 	 * With the other errors, normally the contents hasn't been loaded into
 	 * the buffer, i.e. the buffer is still empty.
 	 */
-	update_buffer_properties = ok || (real_error != NULL &&
-					  real_error->domain == GTK_SOURCE_FILE_LOADER_ERROR &&
-					  real_error->code == GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK);
+	update_file_properties = ok || (real_error != NULL &&
+					real_error->domain == GTK_SOURCE_FILE_LOADER_ERROR &&
+					real_error->code == GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK);
 
-	if (update_buffer_properties && loader->priv->file != NULL)
+	if (update_file_properties && loader->priv->file != NULL)
 	{
 		/* The location is already updated at the beginning of the
 		 * operation.
@@ -1131,12 +1143,29 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 		_gtk_source_file_set_compression_type (loader->priv->file,
 						       loader->priv->auto_detected_compression_type);
 
+		_gtk_source_file_set_externally_modified (loader->priv->file, FALSE);
+		_gtk_source_file_set_deleted (loader->priv->file, FALSE);
+
 		if (g_file_info_has_attribute (loader->priv->info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
 		{
 			GTimeVal modification_time;
 			g_file_info_get_modification_time (loader->priv->info, &modification_time);
 			_gtk_source_file_set_modification_time (loader->priv->file, modification_time);
 		}
+
+		if (g_file_info_has_attribute (loader->priv->info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+		{
+			gboolean readonly;
+
+			readonly = !g_file_info_get_attribute_boolean (loader->priv->info,
+								       G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
+
+			_gtk_source_file_set_readonly (loader->priv->file, readonly);
+		}
+		else
+		{
+			_gtk_source_file_set_readonly (loader->priv->file, FALSE);
+		}
 	}
 
 	reset (loader);
